#include <EEPROM.h>
int offset = 0;  // Correction offset (storage bin)

// Tilt Motor connections
const int Tilt_Motor_RPWM = 5;  // RPWM2
const int Tilt_Motor_LPWM = 6;  // LPWM2

// Steering Motor connections
const int Steer_Motor_RPWM = 11;   // Motor driver input 1 (RPWM)
const int Steer_Motor_LPWM = 3;    // Motor driver input 2 (LPWM)
const int Steer_Motor_Enable = 4;  // Motor driver enable pin (R_EN or L_EN)

// Potentiometer inputs
const int Steer_Pot = A1;           // Control pot
const int Steer_Feedback_Pot = A0;  // Potentiometer for steering feedback
const int buttons = A2;             // analog buttons
const int automan = A3;             // auto / manual switch & button
const int automanLED = 10;          // auto / manual LED indicator
const int Steer_Feedback_Sensor = 12; // Hall sensor for drive gear centered

// Define PID constants
float Kp = 3.0;  // Proportional gain 2.0
float Ki = 0.00;  // Integral gain .03
float Kd = 0.00;  // Derivative gain .05

float previousError = 0;
float integral = 0;
bool modeSwitched = false;      // Track mode switching
bool alignmentDone = false;     // Track if alignment was triggered

void setup() {
  Serial.begin(9600);
  EEPROM.get(0, offset);             // Load stored offset from EEPROM
  pinMode(Tilt_Motor_RPWM, OUTPUT);  //  Tilt
  pinMode(Tilt_Motor_LPWM, OUTPUT);
  pinMode(Steer_Motor_RPWM, OUTPUT);  //  Steering
  pinMode(Steer_Motor_LPWM, OUTPUT);
  pinMode(Steer_Motor_Enable, OUTPUT);
  pinMode(automanLED, OUTPUT);
  pinMode(automan, INPUT_PULLUP);
  pinMode(Steer_Pot, INPUT);  // analog steering control
  // Increase PWM frequency on Timer 2 for pins 3 and 11
  TCCR2B = TCCR2B & 0b11111000 | 0x01;  // ~31.25 kHz PWM frequency on Timer 2
}

void loop() {
  bool mode = digitalRead(automan);  // read switch connected to pin A3
  if (mode == true) {
    digitalWrite(automanLED, HIGH);  // manually turn motor around and tilt as required
    button();
  } else {
    digitalWrite(automanLED, LOW);  // steer with feedback (or align if button 1 pressed)
    steering();
  }
}

int applyCorrection(int reading) {
  return (reading + offset + 1024) % 1024;  // Ensure positive modulo // 1024
}

void button() {
  int b = analogRead(buttons);
  if (b > 82 && b < 205) {
    analogWrite(Steer_Motor_RPWM, 255);  // Button 1 - Steering CW
    analogWrite(Steer_Motor_LPWM, 0);
    digitalWrite(Steer_Motor_Enable, HIGH);  // Button 2 - Steering CCW
  } else if (b > 205 && b < 410) {
    analogWrite(Steer_Motor_RPWM, 0);
    analogWrite(Steer_Motor_LPWM, 255);      // manually move motor
    digitalWrite(Steer_Motor_Enable, HIGH);  // Enable motor
  } else if (b > 410 && b < 614) {           // Button 3 - Tilt UP
    analogWrite(Tilt_Motor_RPWM, 255);
    analogWrite(Tilt_Motor_LPWM, 0);
  } else if (b > 614 && b < 820) {  // Button 4 - Tilt DOWN
    analogWrite(Tilt_Motor_RPWM, 0);
    analogWrite(Tilt_Motor_LPWM, 255);
  } else if (b > 820) {
    analogWrite(Steer_Motor_RPWM, 0);
    analogWrite(Steer_Motor_LPWM, 0);
    analogWrite(Tilt_Motor_RPWM, 0);
    analogWrite(Tilt_Motor_LPWM, 0);  // 5th button does nothing
  } else {
    analogWrite(Steer_Motor_RPWM, 0);
    analogWrite(Steer_Motor_LPWM, 0);
    analogWrite(Tilt_Motor_RPWM, 0);
    analogWrite(Tilt_Motor_LPWM, 0);
  }
}

void align() {
  int rawFeedback = analogRead(Steer_Feedback_Pot);
  int baseline = 512;  // New baseline value
  int lastOffset = offset;
  offset = (baseline - rawFeedback + 1024) % 1024;  //1024
  if (offset != lastOffset) {
    EEPROM.put(0, offset);  // store only if offset changed
    Serial.println("  Calibration complete!");
  }  // Store correction
}

void steering() {
  int c = analogRead(buttons);  // Read button values
  const int alignButtonThreshold = 800;  // Adjust threshold for detecting button press
  
  bool buttonPressed = c < alignButtonThreshold;  // Detect button press

  // Manual alignment in button mode
  if (buttonPressed && digitalRead(automan) == LOW ){  // Only align manually when not in steering mode (automan LOW)
    // Move steering motor until centered sensor goes low
    while (digitalRead(Steer_Feedback_Sensor) == HIGH && digitalRead(Steer_Pot) == HIGH) {  // Check if the centered sensor is not triggered
      analogWrite(Steer_Motor_RPWM, 255);  // Move forward
      analogWrite(Steer_Motor_LPWM, 0);    // No reverse
      digitalWrite(Steer_Motor_Enable, HIGH);
    }

    // Fast stop the motor
    analogWrite(Steer_Motor_RPWM, 0);
    analogWrite(Steer_Motor_LPWM, 0);
    digitalWrite(Steer_Motor_Enable, HIGH);
    delay(500);  // Small delay
    digitalWrite(Steer_Motor_Enable, LOW);
    
    align();  // Perform alignment
    alignmentDone = true;  // Prevent repeated alignment

    // Return to steering after alignment
    return;
  }

  // Auto alignment in steering mode (if automan is HIGH)
  if (digitalRead(automan) == HIGH && buttonPressed) {  // Only auto-align when in steering mode and button is pressed
    // Move steering motor until centered sensor goes low
    while (digitalRead(Steer_Feedback_Sensor) == HIGH) {  // Check if centered sensor is not triggered
      analogWrite(Steer_Motor_RPWM, 175);  // Move forward
      analogWrite(Steer_Motor_LPWM, 0);    // No reverse
      digitalWrite(Steer_Motor_Enable, HIGH);
    }

    // Fast stop the motor
    analogWrite(Steer_Motor_RPWM, 0);
    analogWrite(Steer_Motor_LPWM, 0);
    digitalWrite(Steer_Motor_Enable, LOW);
    delay(1000);  // Small delay

    align();  // Perform alignment
    alignmentDone = true;  // Prevent repeated alignment

    // Return to steering after alignment
    return;
  }

  // Regular steering control (PID)
  if (!buttonPressed) {
    int potValue = analogRead(Steer_Pot);              // Read control potentiometer
    int feedbackValue = analogRead(Steer_Feedback_Pot); // Read feedback potentiometer
    int correctedFeedback = applyCorrection(feedbackValue);  // Apply correction offset

    int controlValue = map(potValue, 0, 1023, 20, 1000);  // Scale pot value

    int error = controlValue - correctedFeedback;  // Calculate steering error
    float P = Kp * error;
    integral += error;
    integral = constrain(integral, -1000, 1000);  // Prevent integral wind-up
    if (abs(error) < 5) integral = 0;  // Reset integral if close to target
    float I = Ki * integral;
    float D = Kd * (error - previousError);
    previousError = error;

    float output = P + I + D;
    int motorSpeed = abs(output);
    motorSpeed = constrain(motorSpeed, 0, 255);  // Bound motor speed

    if (output > 2) {  // Move motor CW
      analogWrite(Steer_Motor_RPWM, motorSpeed);
      analogWrite(Steer_Motor_LPWM, 0);
      digitalWrite(Steer_Motor_Enable, HIGH);
    } else if (output < -2) {  // Move motor CCW
      analogWrite(Steer_Motor_RPWM, 0);
      analogWrite(Steer_Motor_LPWM, motorSpeed);
      digitalWrite(Steer_Motor_Enable, HIGH);
    } else {  // Stop motor
      analogWrite(Steer_Motor_RPWM, 0);
      analogWrite(Steer_Motor_LPWM, 0);
      digitalWrite(Steer_Motor_Enable, LOW);
    }
  }
}











